//**********************************************************************
//    Home Timer -    7- 2-           *
//                                              *
//**********************************************************************
//                                                                     *
//    Filename:	    hometimer1.c                                       *
//    Date:         18.02.2007                                         *
//    File Version: 1.0                                                *
//                                                                     *
//    Author:       picmaniac                                          *
//    Company:      www.pro-radio.ru                                   *
//                                                                     * 
//**********************************************************************
//                                                                     *
//    Files required: 16F628A.h                                        *
//                                                                     *
//**********************************************************************
//                                                                     *
//    Notes:                          *
//           CCS PICC 4.020 + MPLAB 7.42 + Proteus 6.9sp5              *
//                                                                     *
//**********************************************************************

#include <16F628A.h>

#FUSES NOWDT                 	//No Watch Dog Timer
#FUSES XT                    	//Crystal osc <= 4mhz
#FUSES PUT                   	//Power Up Timer
#FUSES NOPROTECT             	//Code not protected from reading
#FUSES NOBROWNOUT            	//No brownout reset
#FUSES MCLR                  	//Master Clear pin enabled
#FUSES NOLVP                 	//No low voltage prgming, B3(PIC16) or B5(PIC18) used for I/O
#FUSES NOCPD                 	//No EE protection

#use delay(clock=4000000,RESTART_WDT)
#use fast_io(A)                 //     
#use fast_io(B)                 //    TRIS

#byte INTCON = 0x0b             //    -    
#byte PORTA = 0x05
#byte PORTB = 0x06
#byte TRISA = 0x85
#byte TRISB = 0x86

#define bit_set(var,bitno) ((var) |= 1 << (bitno))    //   ()
#define bit_clr(var,bitno) ((var) &= ~(1 << (bitno))) //     ()
//---------------------------------------------------------//
// ---  --- //

const unsigned char cD = 0b11101111;   //  " " - RB4
const unsigned char cE = 0b10111111;   //  " " - RB6
const unsigned char cS = 0b11011111;   //  "/"    - RB5

const unsigned char c60SEC = 60;       //    1 
								       //   7- 
const unsigned char cDC7[10] = {0x81, 0xE7, 0x15, 0x45, 0x63, 0x49, 0x09, 0xE5, 0x01, 0x41};
const unsigned char cLED_NUMBER[2] = {0b00001000, 0b00000100};  //   
//---------------------------------------------------------//
// ---  --- //

unsigned char gSTATE;                       //     
enum {ST_READY, ST_SLEEP, ST_ON, ST_SETUP, ST_START, ST_COUNT, \
      ST_SIGNAL, ST_USERSTOP};

unsigned char gLED_DIGITS[2];               //    
unsigned char gLED_COUNTER;                 //   
unsigned char gBUTTONS;                     //  
unsigned char gMIN_D, gMIN_E, gSEC, gQUANT; // 
unsigned char gCSTOP;                       //  (  )
int16 gCW;                                  // 16- 

//---------------------------------------------------------//
// ---   --- //

unsigned char fbuttons (void);
void signalsound (void);

//---------------------------------------------------------//
// ---    --- //
//----------------------------------------//
void delay300us(void) {delay_us(300);}    //  300 
void delay400us(void) {delay_us(400);}    //  400 
void delay100ms(void) {delay_ms(100);}    //  100 
//----------------------------------------//
unsigned char fbuttons(void) //     
{
  unsigned char lTEMP1, lTEMP2, lTEMP3;
  unsigned char lK, lTESTNOW, lTESTOLD, lTEST, lRES;

  lTEMP3 = TRISB;                  //   
  lTEMP2 = PORTA;                  //    
  lTEMP1 = PORTB;

  PORTA = lTEMP2 & 0b00010011;     //  
  TRISB = 0b11111110;              //    
  PORTB = 0b11111110;

  lRES = 0b11111111;               //    - " "

  lTEST = 0;
  for (lK = 0; lK < 9; lK++)
  {
    lTESTNOW = (PORTB | 0b10001111);             //   
    if (lK != 0) lTEST |= (lTESTNOW ^ lTESTOLD); //   
    lTESTOLD = lTESTNOW;                         // (8 )
  }
  if (lTEST == 0) lRES = lTESTNOW; //      
                                   //   8  -     
  TRISB = lTEMP3;                  //   
  PORTB = lTEMP1;                  //    
  PORTA = lTEMP2;
  return lRES;                     //    
}
//----------------------------------------//
void shortsound (void)       //    (!)
{
  const unsigned char lc_how_much = 33; //   (1...255)  . 
  unsigned char lPORT, lK;

  lPORT = PORTA & 0b11111100;
  PORTA = lPORT;                   // . .0    
  for (lk = 0; lK <= lc_how_much; lK++)
  {
    lPORT |= 0b00000001;
    PORTA = lPORT;                 // . .1    
    delay400us();
    lPORT = PORTA & 0b11111100;
    lPORT |= 0b00000010;
    PORTA = lPORT;                 // . .1    
    delay400us();
    lPORT = PORTA & 0b11111100;
    PORTA = lPORT;                 // . .0    
  }
}
//----------------------------------------//
void signalsound (void)      //    
{
  unsigned char lPORT, lI, lJ, lK;

  PORTB = cDC7[0];                 //    "0"

  for (lI = 0; lI <= 20; lI++)
  {
    lPORT = PORTA & 0b11110000;    //  ,   00
    PORTA = lPORT;
    for (lJ = 0; lJ <= 16; lJ++)   //  shortsound
    {
      for (lK = 0; lK <= 40; lK++)
      {
        lPORT |= 0b00000001;
        PORTA = lPORT;
        delay400us();
        lPORT = PORTA & 0b11111100;
        lPORT |= 0b00000010;
        PORTA = lPORT;
        delay400us();
        lPORT = PORTA & 0b11111100;
        PORTA = lPORT;
      } // next lK
      for (lK = 0; lK <= 66; lK++)
      {
        lPORT |= 0b00000001;
        PORTA = lPORT;
        delay300us();
        lPORT = PORTA & 0b11111100;
        lPORT |= 0b00000010;
        PORTA = lPORT;
        delay300us();
        lPORT = PORTA & 0b11111100;
        PORTA = lPORT;
      } // next lK
    } // next lJ

    lPORT = PORTA | 0b00001000;    //    
    PORTA = lPORT;
    delay100ms();
    delay100ms();
    delay100ms();
    if (fbuttons() == cS) break; //   . . "-"
  } // next lI
}
//---------------------------------------------------------//
// --   -- //
//---------------------------------------------------------//
#int_TIMER2
void TIMER2_isr(void) 
{
  if ( ((gSTATE == ST_START) || (gSTATE == ST_COUNT)) && (gQUANT == 250) )
  {
    gQUANT = 0;                             //    250    -
    gSEC--;                                 // ,  1 , 
    if (gSEC == 0xFF)                       //      - 
    {
      gSEC = c60SEC - 1;                    //   59 
      gMIN_E--;					
      if (gMIN_E == 0xFF)                   //      - 
      {
        gMIN_E = 9;                         //   9 
        gMIN_D--;
        if (gMIN_D == 0xFF) gMIN_D = 9;     //       - 
      }                                     //   9 (  )
      gLED_DIGITS[0] = cDC7[gMIN_E];        //  
      gLED_DIGITS[1] = cDC7[gMIN_D];        //  
    }
    if ( (gMIN_D == 0) && (gMIN_E == 1) && (gSEC == 0) ) gSTATE = ST_SIGNAL;
                                            //   -    ""
  } 
  PORTA &= 0b11110011;                      //  
  PORTB = gLED_DIGITS[gLED_COUNTER];        //   -  

                                            //   

  if ( (gSTATE == ST_SETUP) || ( ((gSTATE == ST_START) || (gSTATE == ST_COUNT)) && \
     ((gQUANT & 0b11000000) == 0)) ) PORTA |= cLED_NUMBER[gLED_COUNTER];

              //   -    
  
    else      //    -   
    {
      PORTB = 0b00000001;                   // RB0    

      #asm
        nop                                 //    
        nop                                 //    PORTB
        nop
      #endasm

      if ( (input_state(PIN_B0)) && (gSTATE != ST_SIGNAL) )
      {                                     //  -   ?
        gSTATE = ST_COUNT;                  //  
        if (gCSTOP > 0) gCSTOP--;
      }
          else                              //   
          {
            if ((gSTATE == ST_COUNT) && (gCSTOP < 50))
            {
              gCSTOP ++;
              if (gCSTOP >= 50) gSTATE = ST_USERSTOP;
            }
          }
    }
  gLED_COUNTER++;							//   
  if (gLED_COUNTER >= 2) gLED_COUNTER = 0;	//    -  

  gQUANT++;									//  
  gCW++;
}                                           //   
//-------------------------------------------------------------------//
//_________________________ ________________________//

void main() 
{
   TRISA = 0b00010000;                      //  
   TRISB = 0b00000001;
   PORTA = 0b00010000;
   PORTB = 0b00000001;

   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_1); //  
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,249,16);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   setup_oscillator(False);

   gSTATE = ST_READY;        //   -   Power On

   while (1)                 //  
   {
     switch (gSTATE)         //      ,
     {                       //     -
                             //     (switch-)

       case ST_READY :       //   (,  
                             //      )
         INTCON = 0;
         gLED_COUNTER = 0;
         gMIN_E = 0;
         gMIN_D = 0;
         gLED_DIGITS[0] = cDC7[gMIN_E];
         gLED_DIGITS[1] = cDC7[gMIN_D];
         gSEC = c60SEC;
         gCW = 0;
         gSTATE = ST_SLEEP;  //    ""
         break;              //   switch,    ,  
                             //    switch (gSTATE)

       case ST_SLEEP :       //  "" ()
         
         port_b_pullups(TRUE);     //   PORTB .
         ext_int_edge( H_TO_L );   //      INT

         TRISA = 0b00010000;
         TRISB = 0b00000001;
         PORTA = 0b00010000;
         PORTB = 0b00000001;       //    .0

         INTCON = 0b00010000;      //     INT
         sleep();                  // knock-down :)
         INTCON = 0;               //     INT !

         TRISB = 0b11111110;       //   
         PORTB = 0b11111110;       //   
         delay100ms();             //     

         if ((PORTB | 0b10001111) != 0b11111111) gSTATE = ST_ON;
                             //     -   . ON ("")
         TRISB = 0b00000001;       //   
         break;                    // 


       case ST_ON :                //  "" ()

         PORTB = 0b00000001;
         shortsound();             //   

         setup_timer_2(T2_DIV_BY_1,249,16);
         enable_interrupts(INT_TIMER2);
         enable_interrupts(GLOBAL);    //   TMR2 

         gSTATE = ST_SETUP;        //    ""
         break;


       case ST_SETUP :             //  "" ()

         if (gCW > 2500) {gSTATE = ST_READY; break;}
         //      -    

         disable_interrupts(GLOBAL);
         gBUTTONS = fbuttons();    //  ,    gBUTTONS
         enable_interrupts(GLOBAL);

         switch (gBUTTONS)         //    
         {
           case cD :               //   " " 
             gCW = 0;
             if (gMIN_D < 9) gMIN_D++; else gMIN_D = 0;
             gLED_DIGITS[1] = cDC7[gMIN_D];	//  
             delay100ms();
             delay100ms();
             break;

           case cE :               //   " " 
             gCW = 0;
             if (gMIN_E < 9) gMIN_E++; else gMIN_E = 0;
             gLED_DIGITS[0] = cDC7[gMIN_E];	//  
             delay100ms();
             delay100ms();
             break;

           case cS :               //   "-" 
             gCW = 0;
             if ((gMIN_E == 0) && (gMIN_D == 0))
             {
               disable_interrupts(GLOBAL);  //   
               shortsound();                //    -
               delay400us();                //  
               shortsound();                // ( )
               delay400us();                // ..   
               shortsound();                //  
               enable_interrupts(GLOBAL);

               delay100ms();
             }
               else
               {
                 gCSTOP = 0;
                 gQUANT = 0; 
                 gSTATE = ST_START;         //     -
               }                            //    ""
             break;

           default : delay100ms();          //    

         } // end of switch (gBUTTONS)

         break;


       case ST_USERSTOP :          //  " " 
                                   // ()
         INTCON = 0;
         shortsound();
         gSTATE = ST_READY;        //    ""
         break;


       case ST_SIGNAL :            //  "  "
                                   // ()
         INTCON = 0;
         TRISA = 0;
         PORTA &= 0b11101111;      //  .0  
         signalsound();            //   " "
         TRISA = 0b00010000;       //    Z-
         gSTATE = ST_READY;        //    ""
         break;
         
     } // end of switch (gSTATE)
   } // end of while(1)
} // end of main
